import grok
from zope.component.interfaces import IFactory
from zope.interface import implementedBy
from waeup.interfaces import IHostelContainer, IHostel
from waeup.viewlets import MainArea, Index, Add, FormWrapMixin

class HostelContainer(grok.Container):

    grok.implements(IHostelContainer)
    grok.require('waeup.manageUniversity')

    # A simple counter for ids.
    next_id = 0L

    def addHostel(self, hostel):
        if not IHostel.providedBy(hostel):
            raise TypeError('HostelContainers contain only IHostel instances')
        id = str(self.next_id)
        self[id] = hostel
        while self.next_id in self.keys():
            # Look for next unused int...
            self.next_id += 1
        return id

class HostelContainerFactory(grok.GlobalUtility):
    """A factory for faculty containers.
    """
    grok.implements(IFactory)
    grok.name(u'waeup.HostelContainer')
    title = u"Create a new hostel container.",
    description = u"This factory instantiates new hostel containers."

    def __call__(self):
        return HostelContainer()

    def getInterfaces(self):
        return implementedBy(HostelContainer)


class AddHostelForm(grok.AddForm):
    grok.context(IHostelContainer)
    form_fields = grok.AutoFields(IHostel)
    label = 'Add a hostel'

    @grok.action('Add hostel')
    def addHostel(self, **data):
        hostel = getUtility(IHostel)
        self.applyData(hostel, **data)
        try:
            self.context.addHostel(hostel)
        except DuplicationError:
            self.status = Invalid('The name chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class AddHostel(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddHostelForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IHostelContainer)
    grok.view(Add)
    grok.require('waeup.manageUniversity')

    formview_name = 'addhostelform' # The name of the formview we want
                                    # to be rendered in this viewlet.

